#include <string.h>
#include <stdio.h>

#include "cleanup.h"
#include "io.h"
#include "layoutAll.h"
#include "layout0.h"
#include "node.h"
#include "output.h"
#include "printPS.h"
#include "symbols.h"

extern int yyparse(void);

void usage()
{
  fprintf(stderr,
"lg graph layouter version 0.9 (C) 1995-2006 Armin Biere (biere@jku.at)\n"
"\n"
"-v, --verbose\n"
"-V, --very-verbose\n"
"-h, --help\n"
"-c, --check, --check-only\n"
"-k, --print-koos\n"
"-Y <float>, --ydistance <float>\n"
"-X <float>, --xdistance <float>\n"
"-y <float>, --height <float>\n"
"-x <float>, --width <float>\n"
"-C, --compactify-levels\n"
"-nC, --dont-compactify-levels\n"
"-b <float>, --border-distance <float>\n"
"-e, --encapsulated, --no-border\n"
"--yoffset <float>\n"
"--xoffset <float>\n"
"-s <float>, --scale <float>\n"
"-t <string>, --title <string>\n"
"-W <integer>, --window-size <integer>\n"
"-ECMI <integer>\n"
"--edge-crossings-minimization-iterations-per-level <integer>\n"
"-1 <integer>, --ordering-iterations <integer>\n"
"-2 <integer>, --edge-crosssings-minimization-iterations <integer>\n"
"-3 <integer>, --placing-single-iterations <integer>\n"
"-4 <integer>, --placing-double-iterations <integer>\n"
"--max-priority <integer>\n"
"-d2d <float>, --dummy-to-dummy-distance <float>\n"
"-d2n <float>, --dummy-to-node-distance <float>\n"
"-n2d <float>, --node-to-dummy-distance <float>\n"
"-n2n <float>, --node-to-node-distance <float>\n"
"-draft\n"
"-m, --median, --use-median\n"
"-df <integer>, --dummy-factor <integer>\n"
"--center\n"
"--no-center\n"
"-f2p, --fit-to-page\n"
"-f2g, --fit-to-graph\n"
"-a4\n"
"-letter\n"
"-landscape\n"
"-S, --use-statistics\n"
"-nostats, --dont-use-statistics\n"
"-pup <integer>, --phase-only-up <integer>\n"
"-pdown <integer>, --phase-only-down <integer>\n"
"-pfup <integer>, --phase-first-up <integer>\n"
"-pfdown <integer>, --phase-first-down <integer>\n"
"-o <string>, --output-file <string>\n"
"-i <string>, --input-file <string>\n"
"-nl, --normalize-levels\n"
"-nnl, --dont-normalize-levels\n"
         );
}

int onlyDraft = 0;
char* documentTitle = "<unknown>";

static int opt_atoi(char* s)
{
  int r;

  for(; *s=='0'; s++)
    ;

  for( r=0; *s <= '9' && *s >= '0'; s++)
    r = 10*r + *s - '0';

  if(*s) { usage(); error("while parsing integer command line argument"); }

  return r;
}

static float opt_atof(char* s)
{
  int d;
  float f;

  for(; *s=='0'; s++)
    ;
  for(d =-1, f=0; *s == '.' || ( *s <= '9' && *s >= '0' ); s++)
    {
      if( *s == '.' )
        {
          if( d>= 0)  { usage(); error("multiple dots in float"); }
	  else d=0;
        }
      else
        {
          if(d>=0) d++;
          f = 10*f + *s - '0';
        }
    }
  if(*s)  { usage(); error("while parsing float command line argument"); }
  if(d>=0)
    for(; d; d--)
      f /= 10;
  return f;
}

static int opt_strHasPrefix(char*s, char*m)
{
  int j;
  for(j=0; *m && *s == *m; s++, m++, j++)
    ;
  return !*m ? j : 0;
}

#define OPTION_LOCALS(s,c,i)\
  char **opt_argv = s;\
  int opt_argc = c;\
  int *opt_index = &i;\
  float opt_floatArg;\
  int opt_intArg;\
  char *opt_strArg

static int opt_MF(char **argv, int argc, char *m, float *f, int *index)
{
  char *s = argv[*index] + 1;
  int i = opt_strHasPrefix(s, m);
  if(!i) return 0;
  if(!*(s+i))
    {
      *index = *index + 1;
      if(*index>=argc)  { usage();error("missing float argument"); }
      s = argv[*index];
    }
  else
    {
      s = s + i;
    }
  *f = opt_atof( s );
  return 1;
}

static int opt_MS(char **argv, int argc, char *m, char **r, int *index)
{
  char *s = argv[*index] + 1;
  int i = opt_strHasPrefix(s, m);
  if(!i) return 0;
  if(!s[i])
    {
      *index = *index + 1;
      if(*index>=argc)  { usage();error("missing string argument"); }
      s = argv[*index];
    }
  else
    {
      s = s + i;
    }
  *r = s;
  return 1;
}

static int opt_MI(char **argv, int argc, char *m, int *r, int *index)
{
  char *s = argv[*index] + 1;
  int i = opt_strHasPrefix( s, m);
  if(!i) return 0;
  if(!s[i])
    {
      *index = *index + 1;
      if(*index>=argc)  { usage();error("missing string argument"); }
      s = argv[*index];
    }
  else
    {
      s = s + i;
    }
  *r = opt_atoi(s);
  return 1;
}

#define M(m) ( strcmp(argv[*opt_index]+1, m) == 0 )
#define MF(m) ( opt_MF(opt_argv, opt_argc,  m, &opt_floatArg, opt_index) )
#define MI(m) ( opt_MI(opt_argv, opt_argc, m, &opt_intArg, opt_index) )
#define MS(m) ( opt_MS(opt_argv, opt_argc, m, &opt_strArg, opt_index) )

static int checkPhaseBounds(int i)
{
  if(i<1 || i >= PHASES)
    {
      warning("argument to phase option out of bounds");
      return 0;
    }
  return 1;
}

void parseParameters(int argc, char ** argv)
{
  int i;

  for(i=1; i<argc; i++)
    {
      if(*argv[i]=='-')
	{
	  OPTION_LOCALS(argv,argc,i);

	  if( M("") )
	    ;
	  else if( M("v") || M("-verbose") ) { verboseFlag = 1; }
	  else if( M("V") || M("-very-verbose") ) { verboseFlag = 2; }
	  else if( M("h") || M("?") || M("-help") ) { usage(); exit(0); }
	  else if( M("c") || M("-check") || M("-check-only") )
	    { checkOnlyFlag = 1; }
	  else if( M("k") || M("-print-koos")) { printAll = printKoos; }
	  else if( MF("Y") || MF("-ydistance") ) { yscale = opt_floatArg; }
	  else if( MF("X") || MF("-xdistance") ) { xscale = opt_floatArg; }
	  else if( MF("y") || MF("-height") )
	    { nonStandardHeight = 1; ysize = opt_floatArg; }
	  else if( MF("x") || MF("-width") )
	    { nonStandardWidth = 1; xsize = opt_floatArg; }
	  else if( M("C") || M("-compactify-levels") )
	    { compactifyLevels = 1; }
	  else if( M("nC") || M("-dont-compactify-levels") )
	    { compactifyLevels = 0; }
	  else if( MF("b") || MF("-border-distance") )
	    { xoffset = yoffset = opt_floatArg; }
	  else if( M("e") || M("-encapsulated") || M("-no-border") )
	    { xoffset = yoffset = 0; encapsulated = 1; }
	  else if( MF("-yoffset") ) { yoffset = opt_floatArg; }
	  else if( MF("-xoffset") ) { xoffset = opt_floatArg; }
	  else if( MF("s") || MF("-scale") ) { scale = opt_floatArg; }
	  else if( MS("t") || MS("-title") ) { documentTitle = opt_strArg; }
	  else if( MI("W") || MI("-window-size") )
	    { setWindowSizeTo(opt_intArg); }
	  else if( MI("ECMI") ||
	           MI("-edge-crossings-minimization-iterations-per-level") )
	    { setECMIterationsPerLevel(opt_intArg); }
	  else if(MI("1")||MI("-ordering-iterations") )
	    { phaseIterations[1] = opt_intArg; }
	  else if(MI("2")||MI("-edge-crosssings-minimization-iterations") )
	    { phaseIterations[2] = opt_intArg; }
	  else if(MI("3")||MI("-placing-single-iterations") )
            { phaseIterations[3] = opt_intArg; }
	  else if(MI("4")||MI("-placing-double-iterations") )
            { phaseIterations[4] = opt_intArg; }
	  else if( MI("-max-priority") ) { maxPriority = opt_intArg; }
	  else if( MF("d2d") || MF("-dummy-to-dummy-distance") )
	    { dummy2DummyDistance = opt_floatArg; }
	  else if( MF("d2n") || MF("-dummy-to-node-distance") )
	    { node2DummyDistance = dummy2NodeDistance = opt_floatArg; }
	  else if( MF("n2d") || MF("-node-to-dummy-distance") )
	    { node2DummyDistance = dummy2NodeDistance = opt_floatArg; }
	  else if( MF("n2n") || MF("-node-to-node-distance") )
	    { node2NodeDistance = opt_floatArg; }
	  else if( M("draft") ) { onlyDraft = 1; }
	  else if( M("m") || M("-median") || M("-use-median") )
	    { useMedian(); }
	  else if( MI("df") || MI("-dummy-factor") )
	    { dummyFactor = opt_intArg; }
	  else if( M("-center") ) { centerPage = 1; }
	  else if( M("-no-center") ) { centerPage = 0; }
	  else if( M("f2p") || M("-fit-to-page") ) { fitToPage = 1; }
	  else if( M("f2g") || M("-fit-to-graph") ) { fitToGraph = 1; }
	  else if( M("a4") ) { initPageSize = &a4PageSize; }
	  else if( M("letter") ) { initPageSize = &letterPageSize; }
	  else if( M("landscape") ) { landscape = 1; }
	  else if( M("S") || M("-use-statistics") )
	    { useStatistics = 1; if(!verboseFlag) verboseFlag = 1; }
	  else if( M("nostats") || M("-dont-use-statistics") )
	    { useStatistics = 0; }
	  else if( MI("pup") || MI("-phase-only-up") )
	    { if(checkPhaseBounds(opt_intArg)) phaseOnlyUp[opt_intArg] = 1; }
	  else if( MI("pdown") || MI("-phase-only-down") )
	    { if(checkPhaseBounds(opt_intArg)) phaseOnlyDown[opt_intArg] = 1; }
	  else if( MI("pfup") || MI("-phase-first-up") )
	    { if(checkPhaseBounds(opt_intArg)) phaseFirstUp[opt_intArg] = 1; }
	  else if( MI("pfdown") || MI("-phase-first-down") )
	    { if(checkPhaseBounds(opt_intArg)) phaseFirstUp[opt_intArg] = 0; }
	  else if( MS("o") || MS("-output-file") )
	    {
	      if(output_file_name)
	        error("can only use one output file");
	      output_file_name = opt_strArg;
	    }
	  else if( MS("i") || MS("-input-file") )
	    {
	      if(input_file_name)
	        error("can only use one input file");
	      input_file_name = opt_strArg;
	    }
	  else if( M("nl") || M("-normalize-levels"))
	    { normalizeLevelsFlag = 1; }
	  else if( M("nnl") || M("-dont-normalize-levels"))
	    { normalizeLevelsFlag = 0; }
          else
	    {
	      usage();
	      fprintf(stderr,
	              "*** lg: unknown option `%s'\n",
		      argv[i]);
	      exit(1);
	    }
	}
      else
	{
	  if(input_file_name)
	    error("can only use one input file");
	  input_file_name = argv[i];
	}
    }
  /* defere opening of files until needed */
}

int main(int argc, char** argv)
{
  initPhases();
  parseParameters(argc,argv);
  initAttributes();

  ensureInputFileIsOpen();
  yyparse();
  reverseAllLists();
  completeAttrs();
  checkGraph();

  setLevel();
  normalizeLevels();
  generateDummies();
  setYposToLevelIndex();
  allocateLevels();

  sortLevelsByPredsPos();
  edgeCrossMinimization();
  initNodeDistances();
  placeNodes3();
  placeNodes4();

  calculateOffsetAndSize();
  printAll();

  cleanUp();
  exit(0);
}
