.\" lg (layout graph) man page
.TH lg n "Mon Jun 17 17:49:38 GMT 1996"
.\""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
.SH NAME
lg \- a graph layouter
.SH DESCRIPTION
The graph layouter lg was designed to visualize
BDDs (Binary Decision Diagrams).
But also cyclic and non connected graphs can be handled too.
.P
So the main features of lg are that the user can request
that a node is placed on a specific level or leave
it to lg to allocate levels for nodes and that lg is able
to handle a large number of nodes (>1000).
.P
The output is a layout in Postscript format or a sequence
of nodes and edges with coordinates, attributes and labels.
.\""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
.SH INPUT SYNTAX
The input syntax was designed to be easy to parse and easy to generate.

In general a
.B name
consists of a sequence of alphanumeric characters including `\-' and `_'.
These names can be used to identify nodes and attributes.
Also a name can be used for both purposes.

Attributes can be defined by the user and there is a shallow class hierarchy
for attributes.

The top level attribute is
.B standard
from which all other components inherit their components.
Components of the standard attribute can be overridden too:
.P
.nf
standard { thick }
foo_a { diamond }
foo_b { thin circle }
.fi
.P
This means that all attributes have thick lines (if not overridden like in
the definition of foo_b).
If the user does not give a definition for `standard' then lg chooses default
components for `standard'.
These are printed if the `\-c' option for for pretty printing is used.
.P
Attributes are used to specify the graphical context for drawing
a line or a node.
No distinction between attributes for nodes and edges is made.
A node or an edge has not to be given an attribute.
In this case the
.B default
attribute is used.
It directly inherits from `standard' and can also be redefined by the user.
.P
The keywords for components of attributes are:
\fIbox\fP, \fIcircle\fP, \fIdelta\fP, \fInabla\fP, \fIdiamond\fP,
\fIinvisible\fP, and \fIdot\fP for describing
the shape of a node, \fIdashed\fP, \fIdotted\fP, and \fIsolid\fP
for describing line
styles for both lines surrounding shapes and lines for edges,
\fIblack\fP, \fIred\fP, \fIgreen\fP, \fIblue\fP, \fIyellow\fP as colors,
\fIthick\fP and \fIthin\fP for line
thickness, and finally \fIforward\fP, \fIbackward\fP, and \fInondirected\fP
for describing
the direction of the edge (arrow).
.P
A node definition starts with the name identifying a node followed by
its dream level, attribute, label, and edge definitions (all optional):
.P
.nf
% this is a comment 

a>b<c
a@3'aShortLabel>0>b:foo_a<asdfasdf
b"this is a long label for the node b">foo_a
.fi
.P
defines a graph with the edges a to b, c to a, a to 0,
a second edge from a to b with attribute foo_a,
from asdfasdf to a, and from b to foo_a.

The node a should be placed at level 3, will we printed using attribute foo_a and has 
a short label (without spaces and returns).
Note that the name foo_a is both used for identifying a node and an attribute.
Also you can see that several definitions for the same node are merged. 
But several labels, attributes or levels are not allowed.
.\""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
.SH OPTIONS
.TP
.I "-"
.br
read from sdtin
.TP
.I "-v, --verbose"
.br
write some diagnostics on stdout.
So that you can see what lg is currently doing.
.TP
.I "-V, --very-verbose"
.br
print out some more verbose information.
.TP
.I "-h, --help"
.br
gives you an option summary.
.TP
.I "-c, --check, --check-only"
.br
does only some checks on the graph and prints a pretty version
of the input to stdout. This is also useful for printing the
values of the components of the standard attribute.
.TP
.I "-k, --print-koos"
.br
do not print postscript code but print the coordinates and attributes
of nodes and edges to stdout.
.TP
.I "-Y <float>, --ydistance <float>"
.br
scale the y axis relative to x axis
.TP
.I "-X <float>, --xdistance <float>"
.br
scale the x axis relative to y axis (can be combined with previous option)
.TP
.I "-y <float>, --height <float>"
.br
specify absolute height in (postscript) points (1/72 inch)
.TP
.I "-x <float>, --width <float>"
.br
specify absolute width in (postscript) points (1/72 inch)
.TP
.I "-C, --compactify-levels"
.br
do not use standard sugiyama algorithm which respects previously calculated
position of nodes in one level.
This normally results in more edge crossings but in total width of one level
may decrease.
.TP
.I "-nC, --dont-compactify-levels"
.br
use standard sugiyama algorithm with the goal to minimize edge crossings.
.TP
.I "-b <float>, --border-distance <float>"
.br
specify distance to border in postscript points (default is 72).
.TP
.I "-e, --encapsulated, --no-border"
.br
do not generate full page and set border distance to 0.
.TP
.I "--yoffset <float>"
.br
.TP
.I "--xoffset <float>"
.br
.TP
.I "-s <float>, --scale <float>"
.br
overall scaling of height and width
.TP
.I "-t <string>, --title <string>"
.br
title used in postscript file
.TP
.I "-W <integer>, --window-size <integer>"
.br
use given window size for brute force node permutation algorithm to
minimize edge crossings (maximum window size can be specified at
compilation time and is normally 5).
.TP
.I "-ECMI <integer>"
.br
.TP
.I "--edge-crossings-minimization-iterations-per-level <integer>"
.br
in phase 2 (edge crossings minimization) each level is sweeped times
this number.
.TP
.I "-1 <integer>, --ordering-iterations <integer>"
.br
how many iterations for phase 1 (order nodes per level)
.TP
.I "-2 <integer>, --edge-crosssings-minimization-iterations <integer>"
.br
how many integer for phase 2
.TP
.I "-3 <integer>, --placing-single-iterations <integer>"
.br
how many iterations for phase 3 (place nodes according to predecessors
.B or
successors)
.TP
.I "-4 <integer>, --placing-double-iterations <integer>"
.br
how many iterations for phase 4 (place nodes according to predecessors
.B and
successors). By default this phase is skipped.
.TP
.I "--max-priority <integer>"
.br
in phase 3 or 4 the nodes are placed according to their priority calculated
by the number of connected nodes and whether a node is dummy node.
This number gives the maximal priority.
.TP
.I "-d2d <float>, --dummy-to-dummy-distance <float>"
.br
.TP
.I "-d2n <float>, --dummy-to-node-distance <float>"
.br
.TP
.I "-n2d <float>, --node-to-dummy-distance <float>"
.br
.TP
.I "-n2n <float>, --node-to-node-distance <float>"
.br
.TP
.I "-draft"
.br
only generate a smaller and faster drawable version of the
postscript code
.TP
.I "-m, --median, --use-median"
.br
not implemented yet!
.TP
.I "-df <integer>, --dummy-factor <integer>"
.br
this is the factor the number of connected nodes is multiplied to get
the priority of an dummy node.
.TP
.I "--center"
.br
place the middle of the graph to the center of the page
.TP
.I "--no-center"
.br
place the top left of the graph xoffset/yoffset away from the
top left of the page
.TP
.I "-f2p, --fit-to-page"
.br
use relative scaling factors (see -X and -Y) to fit the graph to
the given page size
.TP
.I "-f2g, --fit-to-graph"
.br
derive page size from dimensions of the graph.
Combined with the -e option this gives the best result for generating
postscript code to be included in other documents.
.TP
.I "-a4"
.br
.TP
.I "-letter"
.br
.TP
.I "-landscape"
.br
.TP
.I "-S, --use-statistics"
.br
.TP
.I "-nostats, --dont-use-statistics"
.br
only enabled when statistics support is compiled in
.TP
.I "-pup <integer>, --phase-only-up <integer>"
.br
given phase is always started at the bottom
.TP
.I "-pdown <integer>, --phase-only-down <integer>"
.br
given phase is always started at the top
.TP
.I "-pfup <integer>, --phase-first-up <integer>"
.br
start given phase at bottom level
.TP
.I "-pfdown <integer>, --phase-first-down <integer>"
.br
start given phase at top level
.TP
.I "-o <string>, --output-file <string>"
.br
.TP
.I "-i <string>, --input-file <string>"
.br
.TP
.I "-nl, --normalize-levels"
.br
delete empty levels (this is the default).
.TP
.I "-nnl, --dont-normalize-levels"
do not delete empty levels.
.br
.\""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
.SH EXAMPLE
The following lines are extracted from a simple BDD library:
.P
.nf
void BDDsimple::visualize()
{
  {
    Visualizer visualizer;
    visualizer.print(this);
  }
  system("lg -12 -nnl -f2g -e -W4 -33 .bdd/tmp.lg > .bdd/tmp.ps");
  system("ghostview .bdd/tmp.ps &");
}
.fi
.P
The combination of 2 phase one and 3 phase 2 iterations did work
very well in practice.
.\""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
.SH AUTHOR
Armin Biere
.br
armin@ira.uka.de
