#include "io.h"
#include <stdlib.h>

char * save_malloc(size_t i)
{
  char* p = (char*)malloc(i);
  if(!p) error("not enough memory");
  return p;
}

char * save_calloc(int n, size_t s)
{
  char* p = (char*)calloc(n,s);
  if(!p) error("not enough memory");
  return p;
}

#ifndef USE_MALLOC
/*---------------------------------------------------------------------------.
 | This is a very very simple memory allocator for small objects of fixed    |
 | (known) size like list nodes etc. It does not reclaim any core memory or  |
 | memory across different sizes.                                            |
 `---------------------------------------------------------------------------*/
#define MAXSIZE 50
#define CHUNKSIZE 1000

static void* freeLists[MAXSIZE];
static void * chunks=0;

/*---------------------------------------------------------------------------.
 | we should test this allocator on a different machine ( != 32bit )         |
 `---------------------------------------------------------------------------*/
#define MAGIC 0x12341234
#endif

void * newCell(int s)
{
#ifdef USE_MALLOC

  return save_malloc(s);

#else
  void* tmp;
  int i,m;

  m = s % sizeof(void*);
  s = s/sizeof(void*);
  if(m) s++;

#if DEBUG
  if(s>=MAXSIZE)
    error("internal newCell: non valid cell size");
#endif

  if(!freeLists[s])
    {
      void * chunk =
        malloc(s*CHUNKSIZE*sizeof(void*) + sizeof(void *));

      *(void **)chunk=chunks;
      chunks=chunk;

      chunk = (void*) (((char*)chunk) + sizeof(void*));

      freeLists[s] = chunk;

      if(!freeLists[s]) error("can not get more core memory");

      for(i=0; i<CHUNKSIZE-1; i++)
	{
	  void** v = (void**)freeLists[s]+s*i;
          *v = &v[s];
#if DEBUG
          for(m=1; m<s; m++)
            v[m] = (void*)MAGIC;
#endif
        }

      ((void**)freeLists[s])[s*(CHUNKSIZE-1)] = 0;
#if DEBUG
      for(m=1; m<s; m++)
        ((void**)freeLists[s])[s*(CHUNKSIZE-1)+m] = (void*)MAGIC;
#endif
    }

  tmp = freeLists[s];
  freeLists[s] = *(void**)freeLists[s];

#if DEBUG
  for(m=1; m<s; m++)
    if(((void**)tmp)[m] != (void*)MAGIC)
      error("internal: used freed memory");
#endif

  return tmp;
#endif
}

void freeCell(void* p, int s)
{

#ifdef USE_MALLOC
  (void) s;
  return free(p);
#else
  int m;

  m = s % sizeof(void*);
  s = s/sizeof(void*);
  if(m) s++;

#if DEBUG
  if(s>=MAXSIZE)
    error("internal in freeCell: non valid cell size");
#endif
  
  *(void**)p = freeLists[s];
  freeLists[s] = p;

#if DEBUG
  for(m=1; m<s; m++)
    ((void**)freeLists[s])[m] = (void*)MAGIC;
#endif
#endif
}

void disposeFreeLists()
{
#ifndef USE_MALLOC
  void * p, * tmp;
  for(p=chunks; p; p=tmp)
    {
      tmp = *(void**)p;
      free(p);
    }
#endif
}
